function f=KinematicCarAnimate(dt, t, l, y1, y2, theta, phi, varargin)
% animates kinematic car using KinematicCarGraphic class
% ** all input vectors must have same length **
% inputs:
%   dt: sample time in s
%   t:  1-d vector of time samples
%   l:  wheelbase length in m for simulated car
%   y1: 1-d vector of horizontal cartesian coordinate of rear axle center
%   y2: 1-d vector of vertical cartesian coordinate of rear axle center
%   theta: 1-d vector of car orientation wrt horizontal
%   phi: 1-d vector of steering angles for simulated car
%   varargin to handle reference car with required arguments:
    %   l:  wheelbase length in m for simulated car
    %   y1: 1-d vector of horizontal cartesian coordinate of rear axle center
    %   y2: 1-d vector of vertical cartesian coordinate of rear axle center
    %   theta: 1-d vector of car orientation wrt horizontal
    %   phi: 1-d vector of steering angles for reference car
% output:
%   f: figure handle

    f=figure;
    set(f,'Units','normalized','OuterPosition',[0 0 0.6 0.6]);
    ax = axes;
    
    set(ax,'DataAspectRatio',[1 1 1],'XGrid','on','YGrid','on');
    xlabel(ax,'y_1 / m')
    ylabel(ax,'y_2 / m')

    % initial state
    car0 = KinematicCarGraphic(ax,l,'r');
    car0.update([y1(1);y2(1)],theta(1),phi(1));
    % car for animating
    car = KinematicCarGraphic(ax,l,'r');
    % plot actual simulated path
    axes(ax);
    line_x=line(y1,y2,'Color','r','LineWidth',1);
    
    switch length(varargin)
        case 0 % simulate only
            axis(ax,[min(y1) max(y1) min(y2) max(y2)]+1.5*l*[-1 1 -1 1]);
            le=legend(line_x,'sim','Location','northwestoutside');
            
        case 5 % reference car too
            lr = reshape(varargin{1},1,[]);
            y1r = reshape(varargin{2},1,[]);
            y2r = reshape(varargin{3},1,[]);
            thetar = reshape(varargin{4},1,[]);
            phir = reshape(varargin{5},1,[]);
            
            axis(ax,[min([y1,y1r]) max([y1,y1r]) min([y2,y2r]) max([y2,y2r])]+1.5*max([l,lr])*[-1 1 -1 1]);
            
            % initial reference state
            carA = KinematicCarGraphic(ax,lr,'b');
            carA.update([y1r(1);y2r(1)],thetar(1),phir(1));
            % final reference state
            carB = KinematicCarGraphic(ax,lr,'b');
            carB.update([y1r(end);y2r(end)],thetar(end),phir(end));
            
            % plot reference path
            axes(ax);
            line_r=line(y1r,y2r,'LineStyle','--','Color','b','LineWidth',1);
            
             le=legend([line_r,line_x],'ref','sim','Location','northwestoutside');
            
        otherwise
            error('5 arguments required for reference car');
    end
    
    % setup timer for animation
    tmr = timer;
    tmr.ExecutionMode = 'fixedRate';
    tmr.Period = dt;
    tmr.UserData = 1; % save index in timer user data for incrementation
    tmr.TasksToExecute = length(t)-1;
    tmr.TimerFcn = @(tim, evt) car.timer_update_fcn(tim, evt, y1, y2, theta, phi);
    tmr.StartFcn = @(tim, evt) car.timer_start_fcn(tim, evt);
    % create replay pushbutton
    bReplay=uicontrol('Parent',f,'Style','pushbutton','String','replay','Units','normalized','Visible','on');
    set(bReplay,'Callback',@(src,event) start(tmr));
    % move button when figure resized
    f.SizeChangedFcn = @(tim, evt) refresh_window(bReplay,le);
    % delete timer when figure closed
    f.DeleteFcn = @(src,event) delete(tmr);
    % perform animation
    start(tmr);   
end

function refresh_window(b,le)
    lep = get(le,'Position');
    set(b,'Position',lep-[0,1.05*lep(4),0,0]);
end