% Kinematic car
% Tutorial 2: open-loop controller

% Systemtheorie und Regelungstechnik
% Saarland University
% March 2021

clear all; close all; clc;

%% add path to common files
addpath(['..' filesep 'common']);

%% Kinematic car parameters
l = 1; % wheelbase length in m

%% simulation parameters
t_end = 10; % simulation end time
dt = 1/20; % sample time of simulated variables (animation frame rate)

%% simulate

% initial conditions
% state defined as x = [y1, y2, theta]' with respect to input (v,phi);
y10 = 0;
y20 = 0;
theta0 = 0;
x0=[y10;y20;theta0];

% generate trajectories
t0=0;
t1=t_end;
r1=PolyRef(t0,t1,y10,5,1,0.5,0,0);
r2=PolyRef(t0,t1,y20,2,0,0,0,0);

% init controller
c = KinematicCarReference(l,r1,r2);

% configure solver
opts = odeset('RelTol',1e-3); % default 1e-3
% opts = odeset();

% run solver
tt = [0:dt:t_end];
% tt = [0,t_end];
[t,x] = ode45(@(t,x) carODE(t,x,l,c), tt, x0, opts);

% format simulation output for readability
[y1,y2,theta] = x2state(x'); % see function below
% evaluate trajectory and controller at desired sample times
eval(r1,t);
eval(r2,t);
eval(c,t);

%% plot results

% spawn a figure
f=figure;
% set figure size and position
set(f,'Units','normalized','OuterPosition',[0 0 1/3 0.6]);

N=5;i=0;
i=i+1;h(i)=subplot(N,1,i);
p{i}=plot(t,c.y1,t,y1);
ylabel('y_1 / m');
i=i+1;h(i)=subplot(N,1,i);
p{i}=plot(t,c.y2,t,y2);
ylabel('y_2 / m');
i=i+1;h(i)=subplot(N,1,i);
p{i}=plot(t,c.th,t,theta);
ylabel('\theta / rad');
i=i+1;h(i)=subplot(N,1,i);
p{i}=plot(t,c.v);
ylabel('v / (m/s)' );
i=i+1;h(i)=subplot(N,1,i);
p{i}=plot(t,c.phi);
ylabel('\phi / rad');
xlabel('t / s')
set(h,'XGrid','on','YGrid','on');
linkaxes(h,'x');
colororder(f,{'b','r'})
legend(h(1),'ref','sim')

%% animate results
f1=KinematicCarAnimate(dt, t, l, y1, y2, theta, c.phi, c.l, c.y1, c.y2, c.th, c.phi);

%% helper functions
function dxdt = carODE(t,x,l,c)
% right hand side of dynamic system state equation (1.4)
% See Flatness-based Control: An Introduction, Shaker Verlag, Feb. 2021
% section 1.3, p.4-6
    
    [~,~,theta] = x2state(x);
    
    % evaluate controller at time t
    eval(c,t);
    
    dxdt = [c.v*cos(theta); ...
            c.v*sin(theta); ...
            c.v*tan(c.phi)/l];
end